/*
 *  Copyright 2020 Alexey Andreev.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.teavm.classlib.impl.tz;

import java.util.TimeZone;
import org.teavm.model.ClassHolder;
import org.teavm.model.ClassHolderTransformer;
import org.teavm.model.ClassHolderTransformerContext;
import org.teavm.model.MethodDescriptor;
import org.teavm.model.MethodHolder;
import org.teavm.model.emit.ProgramEmitter;

public class DateTimeZoneProviderPatch implements ClassHolderTransformer {
    @Override
    public void transformClass(ClassHolder cls, ClassHolderTransformerContext context) {
        if (cls.getName().equals(DateTimeZoneProvider.class.getName())) {
            MethodHolder method = cls.getMethod(new MethodDescriptor("detectTimezone", DateTimeZone.class));
            if (method != null) {
                method.setProgram(null);
                ProgramEmitter pe = ProgramEmitter.create(method, context.getHierarchy());
                pe.constantNull(DateTimeZone.class).returnValue();
            }
        } else if (cls.getName().equals(TimeZone.class.getName())) {
            MethodHolder method = cls.getMethod(new MethodDescriptor("detectTimezone", TimeZone.class));
            if (method != null) {
                method.setProgram(null);
                ProgramEmitter pe = ProgramEmitter.create(method, context.getHierarchy());
                pe.constantNull(TimeZone.class).returnValue();
            }
        }
    }
}
